/*
 * JBoss, Home of Professional Open Source
 * Copyright 2006, JBoss Inc., and others contributors as indicated 
 * by the @authors tag. All rights reserved. 
 * See the copyright.txt in the distribution for a
 * full listing of individual contributors. 
 * This copyrighted material is made available to anyone wishing to use,
 * modify, copy, or redistribute it subject to the terms and conditions
 * of the GNU Lesser General Public License, v. 2.1.
 * This program is distributed in the hope that it will be useful, but WITHOUT A 
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A 
 * PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License,
 * v.2.1 along with this distribution; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, 
 * MA  02110-1301, USA.
 * 
 * (C) 2005-2006,
 * @author Mark Little (mark.little@jboss.com)
 */

package org.jboss.ejb;

import com.arjuna.ats.jta.xa.XidImple;

import com.arjuna.ats.arjuna.common.Uid;

import com.arjuna.ats.arjuna.state.*;

import javax.transaction.xa.XAException;
import javax.transaction.xa.XAResource;
import javax.transaction.xa.Xid;
import org.apache.log4j.Category;
import java.io.*;
import javax.naming.*;

public class ExampleXAResource implements XAResource, Referenceable, Serializable
{
   public ExampleXAResource(String name, boolean exit)
   {
      myLog("ExampleXAResource ("+name+", "+exit+")");

      _name = name;
      _exit = exit;

      if (!_exit)
	  readState();
   }

   /**
    *
    * @param param1 <description>
    * @param param2 <description>
    * @exception javax.transaction.xa.XAException <description>
    */
   public void start(Xid xid, int flags) throws XAException
   {
      myLog("start");
   }

   /**
    *
    * @param param1 <description>
    * @param param2 <description>
    * @exception javax.transaction.xa.XAException <description>
    */
   public void end(Xid xid, int flags) throws XAException
   {
      myLog("end");
   }

  /**
    *
    * @param param1 <description>
    * @param param2 <description>
    * @exception javax.transaction.xa.XAException <description>
    */
   public void commit(Xid xid, boolean onePhase) throws XAException
   {
      myLog("commit,xid=" + xid + ",onePhase=" + onePhase);
      
      if (_exit)
	  Runtime.getRuntime().halt(0);
   }

   /**
    *
    * @param param1 <description>
    * @return <description>
    * @exception javax.transaction.xa.XAException <description>
    */
   public int prepare(Xid xid) throws XAException
   {
      myLog("prepare "+xid);

      writeState(xid);

      return XA_OK;
   }

   /**
    *
    * @param param1 <description>
    * @exception javax.transaction.xa.XAException <description>
    */
   public void rollback(Xid xid) throws XAException
   {
      myLog("rollback");
   }

   /**
    *
    * @param param1 <description>
    * @exception javax.transaction.xa.XAException <description>
    */
   public void forget(Xid xid) throws XAException
   {
      myLog("forget");
   }

   /**
    *
    * @param param1 <description>
    * @return <description>
    * @exception javax.transaction.xa.XAException <description>
    */
   public Xid[] recover(int flag) throws XAException
   {
      myLog("recover");

      Xid[] xids = null;

      if (_xid != null)
      {
	  xids = new Xid[1];
      
	  xids[0] = _xid;

	  System.err.println("expect recovery on "+xids[0]);
      }
      
      return xids;
   }

   /**
    *
    * @param param1 <description>
    * @return <description>
    * @exception javax.transaction.xa.XAException <description>
    */
   public boolean isSameRM(XAResource other) throws XAException
   {
      myLog("isSameRM");
      return(false);
   }

   /**
    *
    * @return <description>
    * @exception javax.transaction.xa.XAException <description>
    */
   public int getTransactionTimeout() throws XAException
   {
      myLog("getTransactionTimeout");
      return 10;
   }

   /**
    *
    * @param param1 <description>
    * @return <description>
    * @exception javax.transaction.xa.XAException <description>
    */
   public boolean setTransactionTimeout(int seconds) throws XAException
   {
      myLog("setTransactionTimeout");
      return true;
   }

  private void myLog (String msg)
  {
    try
    {
      String fileName = "ExampleXAResource"+_name+".log";
      FileWriter fw = new FileWriter(fileName, true);
      PrintWriter pw = new PrintWriter(fw);
      pw.println(msg);
      pw.flush();
      pw.close();
      fw.close();
        System.out.println("ExampleXAResource: "+msg);
    }
    catch (Throwable ex)
    {
      System.out.println("Caught an exception");
      ex.printStackTrace();
    }
  }

  private void writeState (Xid xid)
  {
    try
    {
      String fileName = "ExampleXAResource"+_name+".state";
      FileOutputStream fw = new FileOutputStream(fileName, false);

      OutputObjectState s = new OutputObjectState();
      com.arjuna.ats.jta.xa.XidImple.pack(s, xid);

      System.err.println("**writing XID "+xid);

      byte[] buff = s.buffer();
      
      fw.write(s.buffer());

      fw.flush();
      fw.close();
    }
    catch (Throwable ex)
    {
      System.out.println("Caught an exception");
      ex.printStackTrace();
    }
  }

  private void readState ()
  {
    try
    {
      String fileName = "ExampleXAResource"+_name+".state";
      FileInputStream fr = new FileInputStream(fileName);

      int bufferSize = fr.available();
      byte[] buff = new byte[bufferSize];

      fr.read(buff);

      fr.close();
      
      InputObjectState s = new InputObjectState(new Uid(Uid.nullUid()), "", buff);

      _xid = com.arjuna.ats.jta.xa.XidImple.unpack(s);

      System.err.println("**read Xid "+_xid);
    }
    catch (Throwable ex)
    {
      System.out.println("Caught an exception");
      ex.printStackTrace();
    }
  }

  private void mySleep(long millis)
  {
    myLog("Sleeping " + millis + " milliseconds");
    try
    {
      Thread.sleep(millis);
    }
    catch (Exception ex)
    {
    }
    myLog("Sleep complete");
  }

   public void setReference(Reference _reference)
   {
      myLog("setReference, _reference="+_reference);
      this._reference = _reference;
   }

   public Reference getReference() throws NamingException
   {
      myLog("getReference");
      return _reference;
   }

   private void writeObject(java.io.ObjectOutputStream out) throws IOException
   {
      myLog("writeObject (Serialized)");
   }

   private void readObject(java.io.ObjectInputStream in) throws IOException, ClassNotFoundException
   {
      myLog("readObject (Deserialized");
   }

    private String _name;
    private boolean _exit;
    private Xid _xid;
    private Reference _reference;
    
    private static final long _sleepTime = 20000;    
}
